#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = "heider"
__doc__ = r"""

           Created on 24/01/2022
           """

# TODO: Make instanced Terminal plotter for replotting console contents with new data
