#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = "Christian Heider Nielsen"
__doc__ = r"""

           Created on 14-03-2021
           """

from draugr.writers import Writer


class SqliteWriter(Writer):
    pass  # TODO:
